/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2011 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2011 Sun Microsystems, Inc.
 */
package org.graalvm.visualvm.lib.profiler.api.icons;

import java.awt.Image;
import java.util.Collection;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import org.graalvm.visualvm.lib.profiler.spi.IconsProvider;
import org.openide.util.Lookup;

/**
 * Support for predefined icons and images.
 *
 * @author Jiri Sedlacek
 */
public final class Icons {
    
    /**
     * Returns an Icon instance according to the provided key.
     * 
     * @param key icon key
     * @return Icon instance according to the provided key
     */
    public static Icon getIcon(String key) {
        return getImageIcon(key);
    }
    
    /**
     * Returns an ImageIcon instance according to the provided key.
     * 
     * @param key icon key
     * @return ImageIcon instance according to the provided key
     */
    public static ImageIcon getImageIcon(String key) {
        Image image = getImage(key);
        if (image == null) return null;
        else return new ImageIcon(image);
    }
    
    /**
     * Returns an Image instance according to the provided key.
     * 
     * @param key image key
     * @return Image instance according to the provided key
     */
    public static Image getImage(String key) {
        Collection<? extends IconsProvider> ps = providers();
        for (IconsProvider p : ps) {
            Image image = p.getImage(key);
            if (image != null) return image;
        }
        return null;
    }
    
    /**
     * Returns path to image resource without leading slash according to the provided key.
     * 
     * @param key image key
     * @return path to image resource without leading slash according to the provided key
     */
    public static String getResource(String key) {
        Collection<? extends IconsProvider> ps = providers();
        for (IconsProvider p : ps) {
            String resource = p.getResource(key);
            if (resource != null) return resource;
        }
        return null;
    }
    
    private static Collection<? extends IconsProvider> providers() {
        return Lookup.getDefault().lookupAll(IconsProvider.class);
    }
    
    
    public static interface Keys {}
    
}
