<!--
To change this template, choose Tools | Templates
and open the template in the editor.
-->
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
    </head>
    <body>
        Creates a skeleton VisualVM datasource subtab.
    </body>
</html>
