/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2011 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2011 Sun Microsystems, Inc.
 */
package org.graalvm.visualvm.lib.profiler.api.icons;

/**
 *
 * @author Jiri Sedlacek
 */
public interface GeneralIcons extends Icons.Keys {
    
    public static final String FILTER = "GeneralIcons.Filter"; // NOI18N
    public static final String FILTER_HIGHL = "GeneralIcons.FilterHighl"; // NOI18N
    public static final String SET_FILTER = "GeneralIcons.SetFilter"; // NOI18N
    public static final String SET_FILTER_HIGHL = "GeneralIcons.SetFilterHighl"; // NOI18N
    public static final String CLEAR_FILTER = "GeneralIcons.ClearFilter"; // NOI18N
    public static final String CLEAR_FILTER_HIGHL = "GeneralIcons.ClearFilterHighl"; // NOI18N
    public static final String CLOSE_PANEL = "GeneralIcons.ClosePanel"; // NOI18N
    public static final String FILTER_CONTAINS = "GeneralIcons.FilterContains"; // NOI18N
    public static final String FILTER_NOT_CONTAINS = "GeneralIcons.FilterNotContains"; // NOI18N
    public static final String FILTER_ENDS_WITH = "GeneralIcons.FilterEndsWith"; // NOI18N
    public static final String FILTER_REG_EXP = "GeneralIcons.FilterRegExp"; // NOI18N
    public static final String FILTER_STARTS_WITH = "GeneralIcons.FilterStartsWith"; // NOI18N
    public static final String COLLAPSED_SNIPPET = "GeneralIcons.CollapsedSnippet"; // NOI18N
    public static final String EXPANDED_SNIPPET = "GeneralIcons.ExpandedSnippet"; // NOI18N
    public static final String HIDE_COLUMN = "GeneralIcons.HideColumn"; // NOI18N
    public static final String MAXIMIZE_PANEL = "GeneralIcons.MaximizePanel"; // NOI18N
    public static final String MINIMIZE_PANEL = "GeneralIcons.MinimizePanel"; // NOI18N
    public static final String RESTORE_PANEL = "GeneralIcons.RestorePanel"; // NOI18N
    public static final String SORT_ASCENDING = "GeneralIcons.SortAscending"; // NOI18N
    public static final String SORT_DESCENDING = "GeneralIcons.SortDescending"; // NOI18N
    public static final String POPUP_ARROW = "GeneralIcons.PopupArrow"; // NOI18N
    public static final String ZOOM = "GeneralIcons.Zoom"; // NOI18N
    public static final String ZOOM_IN = "GeneralIcons.ZoomIn"; // NOI18N
    public static final String ZOOM_OUT = "GeneralIcons.ZoomOut"; // NOI18N
    public static final String SCALE_TO_FIT = "GeneralIcons.ScaleToFit"; // NOI18N
    public static final String INFO = "GeneralIcons.Info"; // NOI18N
    public static final String FIND_NEXT = "GeneralIcons.FindNext"; // NOI18N
    public static final String FIND_PREVIOUS = "GeneralIcons.FindPrevious"; // NOI18N
    public static final String SAVE = "GeneralIcons.Save"; // NOI18N
    public static final String SAVE_AS = "GeneralIcons.SaveAs"; // NOI18N
    public static final String SAVE_VIEW = "GeneralIcons.SaveView"; // NOI18N
    public static final String EXPORT = "GeneralIcons.Export"; // NOI18N
    public static final String DETACH = "GeneralIcons.Detach"; // NOI18N
    public static final String START = "GeneralIcons.Start"; // NOI18N
    public static final String PAUSE = "GeneralIcons.Pause"; // NOI18N
    public static final String RERUN = "GeneralIcons.Rerun"; // NOI18N
    public static final String RESUME = "GeneralIcons.Resume"; // NOI18N
    public static final String STOP = "GeneralIcons.Stop"; // NOI18N
    public static final String EMPTY = "GeneralIcons.Empty"; // NOI18N
    public static final String ERROR = "GeneralIcons.Error"; // NOI18N
    public static final String FIND = "GeneralIcons.Find"; // NOI18N
    public static final String SLAVE_DOWN = "GeneralIcons.SlaveDown"; // NOI18N
    public static final String SLAVE_UP = "GeneralIcons.SlaveUp"; // NOI18N
    public static final String UPDATE_AUTO = "GeneralIcons.UpdateAuto"; // NOI18N
    public static final String PIE = "GeneralIcons.Pie"; // NOI18N
    public static final String UPDATE_NOW = "GeneralIcons.UpdateNow"; // NOI18N
    public static final String BUTTON_ATTACH = "GeneralIcons.ButtonAttach"; // NOI18N
    public static final String BUTTON_RUN = "GeneralIcons.ButtonRun"; // NOI18N
    public static final String UP = "GeneralIcons.Up"; // NOI18N
    public static final String DOWN = "GeneralIcons.Down"; // NOI18N
    public static final String FORWARD = "GeneralIcons.Forward"; // NOI18N
    public static final String BACK = "GeneralIcons.Back"; // NOI18N
    public static final String SETTINGS = "GeneralIcons.Settings"; // NOI18N
    public static final String JAVA_PROCESS = "GeneralIcons.JavaProcess"; // NOI18N
    public static final String FOLDER = "GeneralIcons.Folder"; // NOI18N
    public static final String BADGE_ADD = "GeneralIcons.BadgeAdd"; // NOI18N
    public static final String BADGE_REMOVE = "GeneralIcons.BadgeRemove"; // NOI18N
    public static final String RENAME = "GeneralIcons.Rename"; // NOI18N
    public static final String MATCH_CASE = "GeneralIcons.MatchCase"; // NOI18N
    public static final String ADD = "GeneralIcons.Add"; // NOI18N
    public static final String EDIT = "GeneralIcons.Edit"; // NOI18N
    public static final String REMOVE = "GeneralIcons.Remove"; // NOI18N
    public static final String HIGHLIGHT_RESULTS = "GeneralIcons.HighlightResults"; // NOI18N
    
}
