/*
 * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.graalvm.visualvm.modules.appui.toolbar;

import javax.swing.JComponent;
import org.graalvm.visualvm.lib.ui.UIUtils;
import org.graalvm.visualvm.lib.ui.components.ProfilerToolbar;
//import org.openide.util.lookup.ServiceProvider;

/**
 * NOTE: disabled since the ProfilerToolbar is transparent by default
 *
 * @author Tomas Hurka
 */
//@ServiceProvider(service=ProfilerToolbar.Provider.class)
public class ToolbarProvider extends ProfilerToolbar.Provider {

    @Override
    public ProfilerToolbar create(boolean showSeparator) {
        return new VisualVMImpl(showSeparator);
    }
    
    
    private static class VisualVMImpl extends ProfilerToolbar.Impl {
        
        private VisualVMImpl(boolean showSeparator) {
            super(showSeparator);
            
            JComponent comp = component;
            if (showSeparator) comp = (JComponent)comp.getComponent(0);
            
//            comp.setOpaque(false);
            comp.setOpaque(true);
            comp.setBackground(UIUtils.getProfilerResultsBackground());
        }
        
    }
    
}
