/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.graalvm.visualvm.lib.profiler.snaptracer.logs;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.NavigableMap;
import java.util.TreeMap;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import org.openide.filesystems.FileObject;

/** Reads log records from file.
 *
 * @author Tomas Hurka
 */
public final class LogReader {
 
    private static final Logger LOG = Logger.getLogger(LogRecords.class.getName());

    private FileObject logFile;
//    private int records;
//    private long startTime;
    private NavigableMap<Long,LogRecord> recordList;

    public LogReader(FileObject f) {
        logFile = f;
        recordList = new TreeMap();
    }


    public void load() throws IOException {
        InputStream is = new BufferedInputStream(logFile.getInputStream(),32768);
        try {
            LogRecords.scan(is, new LogHandler());
        } finally {
            is.close();
        }
    }

    public LogRecord getRecordFor(long time) {
        Map.Entry<Long,LogRecord> entry = recordList.floorEntry(new Long(time));
        
        if (entry != null) {
            return entry.getValue();
        }
        return null;
    }
    
    class LogHandler extends Handler {

        @Override
        public void publish(LogRecord record) {
//            System.out.println("Record "+ records++);
//            if (startTime == 0) {
//                startTime = record.getMillis();
//                System.out.println("Start date: "+new Date(startTime));
//            } else {
//                System.out.println("Time: "+(record.getMillis()-startTime));
//            }
//            System.out.println(record.getMessage());
            recordList.put(new Long(record.getMillis()), record);
        }

        @Override
        public void flush() {
//            System.out.println("Flush");
        }

        @Override
        public void close() throws SecurityException {
//           System.out.println("Close");
        }

    }
}
