/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.graalvm.visualvm.lib.profiler.heapwalk.details.jdk;

import java.lang.ref.Reference;
import org.graalvm.visualvm.lib.jfluid.heap.Heap;
import org.graalvm.visualvm.lib.jfluid.heap.Instance;
import org.graalvm.visualvm.lib.profiler.heapwalk.details.spi.DetailsProvider;
import org.graalvm.visualvm.lib.profiler.heapwalk.details.spi.DetailsUtils;
import org.graalvm.visualvm.lib.profiler.heapwalk.model.BrowserUtils;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Jiri Sedlacek
 */
@ServiceProvider(service=DetailsProvider.class)
public final class ReferenceDetailsProvider extends DetailsProvider.Basic {

    private static final String FEEBLE_REF_MASK = "com.oracle.svm.core.heap.FeebleReference+";  // NOI18N
    
    public ReferenceDetailsProvider() {
        super(Reference.class.getName() + "+", FEEBLE_REF_MASK);                // NOI18N
    }
    
    public String getDetailsString(String className, Instance instance, Heap heap) {
        if (FEEBLE_REF_MASK.equals(className)) {
            Object value = instance.getValueOfField("rawReferent");             // NOI18N
            if (value instanceof Instance) {
                return getRefDetail(value, heap);
            }
        } else {
            Object value = instance.getValueOfField("referent");                // NOI18N
            if (value instanceof Instance) {
                return getRefDetail(value, heap);
            }
            value = instance.getValueOfField("feeble");                         // NOI18N
            if (value instanceof Instance) {
                return DetailsUtils.getInstanceString((Instance) value, heap);
            }
            value = instance.getValueOfField("bootImageStrongValue");           // NOI18N
            if (value instanceof Instance) {
                return getRefDetail(value, heap);
            }
        }
        return null;
    }

    private String getRefDetail(Object value, Heap heap) {
        Instance i = (Instance)value;
        // TODO: can create cycle?
        String s = DetailsUtils.getInstanceString(i, heap);
        s = s == null ? "#" + i.getInstanceNumber() : ": " + s;                 // NOI18N
        return BrowserUtils.getSimpleType(i.getJavaClass().getName()) + s;
    }
    
}
